@@ -16,5 +16,5 @@
   args["lazy"] <- list(lazy)
   options(future.disposable = args)
 
-  eval(fassignment, envir=envir)
+  eval(fassignment, envir = envir, enclos = baseenv())
 }
