@@ -36,20 +36,20 @@
 #' @section Implemented evaluation strategies:
 #' \itemize{
 #'  \item{\code{\link{sequential}}:}{
-#'    Resolves futures sequentially in the current R process.
+#'    Resolves futures sequentially in the current \R process.
 #'  }
 #'  \item{\code{\link{transparent}}:}{
-#'    Resolves futures sequentially in the current R process and
+#'    Resolves futures sequentially in the current \R process and
 #'    assignments will be done to the calling environment.
 #'    Early stopping is enabled by default.
 #'  }
 #'  \item{\code{\link{multisession}}:}{
 #'    Resolves futures asynchronously (in parallel) in separate
-#'    R sessions running in the background on the same machine.
+#'    \R sessions running in the background on the same machine.
 #'  }
 #'  \item{\code{\link{multicore}}:}{
 #'    Resolves futures asynchronously (in parallel) in separate
-#'    \emph{forked} R processes running in the background on
+#'    \emph{forked} \R processes running in the background on
 #'    the same machine.  Not supported on Windows.
 #'  }
 #'  \item{\code{\link{multiprocess}}:}{
@@ -58,15 +58,15 @@
 #'  }
 #'  \item{\code{\link{cluster}}:}{
 #'    Resolves futures asynchronously (in parallel) in separate
-#'    R sessions running typically on one or more machines.
+#'    \R sessions running typically on one or more machines.
 #'  }
 #'  \item{\code{\link{remote}}:}{
-#'    Resolves futures asynchronously in a separate R session
+#'    Resolves futures asynchronously in a separate \R session
 #'    running on a separate machine, typically on a different
 #'    network.
 #'  }
 #' }
-#'
+#' 
 #' Other package may provide additional evaluation strategies.
 #' Notably, the \pkg{future.batchtools} package implements a
 #' type of futures that will be resolved via job schedulers
@@ -74,6 +74,10 @@
 #' (HPC) clusters, e.g. LSF, Slurm, TORQUE/PBS, Sun Grid Engine,
 #' and OpenLava.
 #'
+#' To "close" any background workers (e.g. \code{multisession}), change
+#' the plan to something different; \code{plan(sequential)} is recommended
+#' for this.
+#'
 #' @section For package developers:
 #' Please refrain from modifying the future strategy inside your packages /
 #' functions, i.e. do not call \code{plan()} in your code.  Instead, leave
@@ -92,6 +96,18 @@
 #'   [...]
 #' }
 #'
+#' @section Using plan() in scripts and vignettes:
+#' When writing scripts or vignettes that uses futures, try to place any
+#' call to \code{plan()} as far up (as early on) in the code as possible.  
+#' This will help users to quickly identify where the future plan is set up
+#' and allow them to modify it to their computational resources.
+#' Even better is to leave it to the user to set the \code{plan()} prior to
+#' \code{source()}:ing the script or running the vignette.
+#' If a \file{\link{.future.R}} exists in the current directory and / or in
+#' the user's home directory, it is sourced when the \pkg{future} package is
+#' \emph{loaded}.  Because of this, the \file{.future.R} file provides a
+#' convenient place for users to set the \code{plan()}.
+#'
 #' @export
 plan <- local({
   defaultStrategy <- structure(sequential, call = substitute(plan(sequential)))
@@ -108,7 +124,7 @@ plan <- local({
 
   plan_init <- function() {
     evaluator <- stack[[1L]]
-    init <- attr(evaluator, "init")
+    init <- attr(evaluator, "init", exact = TRUE)
     if (identical(init, TRUE)) {
       debug <- getOption("future.debug", FALSE)
       if (debug) {
@@ -123,7 +139,8 @@ plan <- local({
       stack[[1L]] <<- evaluator
 
       ## Create dummy future to trigger setup (minimum overhead)
-      f <- evaluator(NA, globals = FALSE, lazy = FALSE)
+      f <- evaluator(NA, label = "future-plan-test", 
+                     globals = FALSE, lazy = FALSE)
 
       ## Cleanup, by resolving it
       ## (otherwise the garbage collector would have to do it)
@@ -269,7 +286,7 @@ plan <- local({
       for (kk in seq_along(newStack)) {
         strategy <- newStack[[kk]]
         ## Skip if already has a call attibute
-        if (!is.null(attr(strategy, "call"))) next
+        if (!is.null(attr(strategy, "call", exact = TRUE))) next
         ## Attach call
         attr(strategy, "call") <- call
         newStack[[kk]] <- strategy
@@ -312,13 +329,15 @@ print.future <- function(x, ...) {
   class <- setdiff(class(x), c("FutureStrategy", "tweaked", "function"))
   s <- sprintf("%s:", class[1])
   specs <- list()
-  args <- deparse(args(x))
+  args <- deparse(args(x), width.cutoff = 500L)
   args <- args[-length(args)]
   args <- gsub("(^[ ]+|[ ]+$)", "", args)
   args <- paste(args, collapse = " ")
   specs$args <- args
   specs$tweaked <- inherits(x, "tweaked")
-  specs$call <- deparse(attr(x, "call"))
+  specs$call <- paste(deparse(attr(x, "call", exact = TRUE), 
+                              width.cutoff = 500L),
+                      collapse="")
   specs <- sprintf("- %s: %s", names(specs), unlist(specs))
   s <- c(s, specs)
   s <- paste(s, collapse = "\n")
@@ -339,13 +358,15 @@ print.FutureStrategyList <- function(x, ...) {
     class <- setdiff(class(x_kk), c("tweaked", "function"))
     s_kk <- sprintf("%d. %s:", kk, class[1])
     specs <- list()
-    args <- deparse(args(x_kk))
+    args <- deparse(args(x_kk), width.cutoff = 500L)
     args <- args[-length(args)]
     args <- gsub("(^[ ]+|[ ]+$)", "", args)
     args <- paste(args, collapse = " ")
     specs$args <- args
     specs$tweaked <- inherits(x_kk, "tweaked")
-    specs$call <- deparse(attr(x_kk, "call"))
+    specs$call <- paste(deparse(attr(x_kk, "call", exact = TRUE), 
+                                width.cutoff = 500L),
+                        collapse = "")
     specs <- sprintf("   - %s: %s", names(specs), unlist(specs))
     s <- c(s, s_kk, specs)
   }
